import { ClientOptions, Collection, Message, Snowflake } from "discord.js";
import { CommandClient } from "./commandclient";
import { Player } from "./models/player";
import { Battle } from "./battle";
import { Sequelize } from "sequelize";
import { Unit } from "./models/unit";
import { CHARACTERS } from "./characters";

export class Challenge {
    constructor (player1: Player, player2: Player, message: Message) {
        this.player1 = player1;
        this.player2 = player2;
        this.message = message
        this.id = player1.user_id + player2.user_id;
    }
    
    player1: Player
    player2: Player
    message: Message
    id: string
    created = Date.now()
}

export class ActiveBattle {
    constructor (battle: Battle, message: Message) {
        this.battle = battle;
        this.message = message;
    }

    sendRound () {

    }

    static async create (battle: Battle) {
        const activeBattle = new ActiveBattle(battle, (await battle.channel.send('initial battle state')));
        return activeBattle;
    }

    battle: Battle
    message: Message
}

export class CBClient extends CommandClient {
    constructor (options: ClientOptions, db: Sequelize) {
        super(options, db);

        this.on('interactionCreate', async (int) => {
            if (!int.isButton() || !int.channel || int.channel.isDMBased()) return;

            const defaultUnit = await Unit.findOne({ where: { id: 0 } }) as Unit;

            const challenge = this.CHALLENGES.get(int.customId);
            if (!challenge || challenge.created+60_000 < Date.now()) {
                int.reply("That challenge has expired!");
                return;
            }

            if (int.user.id != challenge.player2.user_id) {
                int.reply("You can't accept someone else's challenge!");
                return;
            }

            if (this.ACTIVE_BATTLES.find(e => {
                return [challenge.player1.user_id, challenge.player2.user_id].includes(e.battle.player1.user_id)
                || [challenge.player1.user_id, challenge.player2.user_id].includes(e.battle.player2.user_id);
            })) {
                int.reply("A participant is already in a battle!");
                return;
            }

            await challenge.message.delete();
            this.CHALLENGES.delete(challenge.id);

            const team1: [Unit, Unit, Unit] = [
                await Unit.findOne({ where: { id: challenge.player1.slot1 } }) || defaultUnit,
                await Unit.findOne({ where: { id: challenge.player1.slot2 } }) || defaultUnit,
                await Unit.findOne({ where: { id: challenge.player1.slot3 } }) || defaultUnit
            ]
            const team2: [Unit, Unit, Unit] = [
                await Unit.findOne({ where: { id: challenge.player2.slot1 } }) || defaultUnit,
                await Unit.findOne({ where: { id: challenge.player2.slot2 } }) || defaultUnit,
                await Unit.findOne({ where: { id: challenge.player2.slot3 } }) || defaultUnit
            ]
            const battle = new Battle(this, int.channel, challenge.player1, challenge.player2, team1, team2);
            const activeBattle = new ActiveBattle(battle,
                await int.channel.send({
                    content: 'wooo'
                }
            ));
            this.ACTIVE_BATTLES.push();
        });
    }

    readonly CHALLENGES: Collection<string, Challenge> = new Collection()
    readonly ACTIVE_BATTLES: ActiveBattle[] = []

    static async createPlayer(user_id: Snowflake) {
        return Player.create({
            user_id: user_id,
            start: Math.floor(Date.now()/1000)
        });
    }

    static async initializePlayer(user_id: Snowflake) {
        Unit.create({ user_id: user_id, character_id: CHARACTERS.randomKey()! });
        Unit.create({ user_id: user_id, character_id: CHARACTERS.randomKey()! });
        Unit.create({ user_id: user_id, character_id: CHARACTERS.randomKey()! });
        return CBClient.createPlayer(user_id);
    }

    static async findOrCreatePlayer(user_id: Snowflake) {
        const player = await Player.findOne({
            where: {
                user_id: user_id
            }
        });

        if (!player) return await CBClient.initializePlayer(user_id);
        return player;
    }

    static async spendOnPlayer(player: Player, amount: number) {
        const currency = Date.now() - player.start - player.spent;
        
        if (currency - amount < 0) return false;
        await player.update("spent", player.spent+amount);
        return true;
    }

    static async grantMoneyToPlayer(player: Player, amount: number) {
        await player.update("spent", player.spent-amount);
        return;
    }
}